#ifndef _COMMANDS_H_
#define _COMMANDS_H_

/* command format:

1   byte : command number (see below)
n   bytes: command parameters (data)

changes from original TBDM:
  speed has higher resolution
  go and step commands cannot have optional parameters because of different command format
  block commands have smaller size limit and transfer data as bytes rather than words for the HC12 platform
  write block commands now have data count parameter in addition to address
  version command now returns differently formated number
  string descriptor is now returned by USB directly and the command does not exist anymore
  echo commands have been removed, use GET_VER instead
  get status has slightly extended format

Commands not implemented compared to original TBDM:

CMD_WRITE_REGS        - debuggers typically need write access to individual registers and thus having 
                        an aggregate command is not of a substantial benefit (how often are the registers
                        written anyway...)
CMD_WRITE_BLOCK_FAST  - fast block operations are only available in active background mode. Their use
                        for debuggers is therefore limited as the debugger needs to have access to memory
                        independant of the target state
CMD_READ_BLOCK_FAST     moreover the BDM transfer time is comparable to low speed USB on faster
                        targets and the faster BDM operation does not neccessaily bring a huge advantage
                        here at 2MHz BDM frequency the saving for 128 byte block would be slightly less 
                        than 64 transfers * 16 bit of address * bit time = 64 * 16 * 16/2 us = 8192us
                           while the overall transfer length with HW word writes is at least
                           64 tfrs * 40 bits * bit time + USB transfer time 
                             = 64 * 40 * 16/2 us + 128/8 * 1000us = 36480us
                           i.e. the saving would be less than 22% and smaller with crystals > 4MHz
data format:
      all 16-bit word data is transferred in intel (little) endian, 
        i.e. LSB on lower and MSB on higher address
*/

/*
 * If command fails, the device responds with command code CMD_FAILED
 * If command succeeds, the device responds with the same command number followed by any results
 *     as appropriate 
*/

#define MAX_PACKET_SIZE       (13*8) // Maximum USB packet size - entire packet! - Limited by Z-Page RAM - multiple of 8 is nice
                                     // since transfers may be in 8-byte pieces.
//#define MAX_DATA_SIZE              // No longer used since actual data size depends on status/info overhead

/* System related commands */
#define CMD_OK                0  // deprecated
#undef  CMD_ERR_SUM              // 1 deprecated, USB has CRC16 in place
#define CMD_FAILED            2  // command execution failed (incorrect parameters, 
                                 // 2 target not responding, etc.)
#define CMD_UNKNOWN           3  // unknown command

#define CMD_DEBUG             8  // Debugging commands (parameter determines actual command)
#define CMD_OPTION            10 // Set/clear various options (only has affect after next CMD_SET_TARGET)

/* TurboBdmLight related commands */
#undef  CMD_GET_DESC             // 10 deprecated
#undef  CMD_GET_SWSER            // 11 deprecated
#define CMD_GET_VER           12 // returns 16 bit HW/SW version number, (major & minor revision
                                 //       in BCD in each byte - HW in MSB, SW in LSB; intel endianism) 
#define CMD_GET_LAST_STATUS   13 // returns status of the previous command
#define CMD_SET_BOOT          14 // request bootloader firmware upgrade on next power-up, parameters: 'B','O','O','T', returns: none

/* BDM related commands */
#define CMD_SET_TARGET        30 // set target, 8bit parameter: 00=HC12/HCS12(default), 01=HCS08
#define CMD_CONNECT           31 // try to connect to the target
#undef  CMD_READ_SPEED           // 32 deprecated, speed has higher resolution now
#define CMD_RESET             33 // 8bit parameter: 0=reset to Special Mode, 1=reset to Normal mode
#define CMD_GET_STATUS        34 // returns 16bit status word: 
                                 //   bit0 - ACKN,
                                 //   bit1 - target was reset (this bit is cleared after reading), 
                                 //   bit2 - always 0, 
                                 //   bit4-3 - comm status: 00=NOT CONNECTED, 01=SYNC, 10=GUESS, 
                                 //                                   11=USER SUPPLIED
#define CMD_READ_BD           35 // parameter: 16-bit address, returns 8-bit value read from address;
                                 //   on HCS08 it returns the 8-bit BDCSCR register, parameters are none
#define CMD_WRITE_BD          36 // parameter: 16-bit address, 8-bit value to write;
                                 //   on HCS08 it writes the BDCSCR register, 
                                 //   parameter is only the 8-bit value to write
#undef  CMD_GO                   // deprecated
#undef  CMD_STEP                 // deprecated
#define CMD_HALT              39 // stop the CPU and bring it into background mode
#define CMD_SET_SPEED         40 // deprecated
#define CMD_READ_SPEED1       41 // read speed of the target: returns 16-bit tick count
#define CMD_GO1               42 // start code execution
#define CMD_STEP1             44 // perform single step
#define CMD_SET_SPEED1        46 // sets-up the BDM interface for a new bit rate & tries
                                 //   to enable ackn feature, parameter: 
                                 //   16-bit word = number of 16.6ns ticks per SYNC

/* CPU related commands */
#define CMD_READ_8            50 // parameter 16bit address, returns 8bit value read from address
#define CMD_READ_16           51 // parameter 16bit address, returns 16bit value read from address
#undef  CMD_READ_BLOCK           // 52 deprecated
#define CMD_READ_REGS         53 // reads registers, returns 16bit values: 
                                 //    HC/S12(X): PC, SP, IX, IY, D(B:A), CCR; 
                                 //    HCS08: PC, SP, H:X, A, CCR; upper bytes are 0 when not used
#undef  CMD_READ_BLOCK_FAST      // 54 deprecated
#define CMD_READ_BLOCK1       55 // parameter 16bit address, 8bit count of bytes to read,
                                 //    returns block of bytes from given address, 
                                 //    count MUST be <=MAX_DATA_SIZE

#define CMD_WRITE_8           60 // parameter 16bit address, 8bit value to write
#define CMD_WRITE_16          61 // parameter 16bit address, 16bit value to write
#undef  CMD_WRITE_BLOCK          // 62 deprecated
#undef  CMD_WRITE_REGS           // 63 deprecated
#undef  CMD_WRITE_BLOCK_FAST     // 64 deprecated
#define CMD_WRITE_BLOCK1      65 // parameters: 16bit address, 8bit count of bytes, 
                                 //   block of bytes to write, 
                                 //   size of data block can be up to MAX_DATA_SIZE

/*
 * HC12 CPU individual register write commands 
 * (added for debugger interfaces which need access to individial registers rather than whole block)
 */
#define CMD_WRITE_REG_PC      80 // parameter: 16-bit PC value
#define CMD_WRITE_REG_SP      81 // parameter: 16-bit SP value
#define CMD_WRITE_REG_X       82 // parameter: 16-bit IX (H:X) value
#define CMD_WRITE_REG_Y       83 // parameter: 16-bit IY value
#define CMD_WRITE_REG_D       84 // parameter: 16-bit B:A (x:A) value
#define CMD_WRITE_REG_CCR     85 // parameter: 16-bit CCR (x:CCR) value

// Coldfire V1
#define CMD_CF_READ_CSR2      86
#define CMD_CF_READ_CSR3      87
#define CMD_CF_WRITE_CSR2     88
#define CMD_CF_WRITE_CSR3     89
#define CMD_CF_READ_REG       90
#define CMD_CF_READ_CREG      91
#define CMD_CF_READ_DREG      92
#define CMD_CF_READ_XCSR      93
#define CMD_CF_WRITE_REG      94
#define CMD_CF_WRITE_CREG     95
#define CMD_CF_WRITE_DREG     96
#define CMD_CF_READ_MEM       97
#define CMD_CF_WRITE_MEM      98
#define CMD_CF_WRITE_XCSR     99

// S08
#define CMD_READ_STATUS       100
#define CMD_WRITE_CONTROL     101
#define CMD_WRITE_BKPT        102
#define CMD_READ_BKPT         103
 
//RS08
#define CMD_READ_SPC          104
#define CMD_WRITE_SPC         105
#define CMD_READ_CCR_PC       106
#define CMD_WRITE_CCR_PC      107
#define CMD_READ_BLOCK        108
#define CMD_WRITE_BLOCK       109
#define CMD_MASS_ERASE        110
#define CMD_VPPOFF            111
#define CMD_RS08_POG          112
#define CMD_FLASH_DLSTART     113
#define CMD_FLASH_DLEND       114
#define TCMD_NEXT_DLDATA      115 

// 1. WRITE_16 and WRITE_16 work on HC/S12(X) only

/* TBD: special commands specific to HCS08 (breakpoints)
        may need separate commands for writing X and H on S08

   Block commands may need to be changed to include a checksum to protect the data
   in case they prove to be unreliable

   Then an additional improvement would be to only clear a command exec failure after a special
   command (CMD_CLEAR_ERROR).  This would enable the user to execute several commands and only 
   then check whether any of them has failed.  It does not make sense at this moment as the 
   commands do not check their parameters very much...

*/

/* Error code from BDM routines*/
enum {
  BDM_RC_OK                      = 0,     // No error
  BDM_RC_BUSY                    = 1,     // Busy with last command - try again
  BDM_RC_OVERRUN                 = 2,     // New command before previous command completed
  BDM_RC_ILLEGAL_PARAMS          = 3,     // Illegal parameters to command
  BDM_RC_ILLEGAL_COMMAND         = 4,     // Illegal (unknown) command (may be in wrong target mode)
  BDM_RC_NO_CONNECTION           = 5,     // No connection to target
  
  BDM_RC_UNKNOWN_TARGET          = 11,    // Target not supported
  BDM_RC_NO_TX_ROUTINE           = 12,    // No Tx routine available at measured BDM communication speed
  BDM_RC_NO_RX_ROUTINE           = 13,    // No Rx routine available at measured BDM communication speed
  BDM_RC_BDM_EN_FAILED           = 14,    // Failed to enable BDM mode in target
  BDM_RC_RESET_TIMEOUT           = 15,    // RESET signal failed to rise/fall
  BDM_RC_BKGD_TIMEOUT            = 16,    // BKGD signal failed to rise/fall
  BDM_RC_SYNC_TIMEOUT            = 17,    // No response to SYNC sequence
  BDM_RC_UNKNOWN_SPEED           = 18,    // Communication speed is not known
  BDM_RC_WRONG_PROGRAMMING_MODE  = 19,    // Flash programmin when in wrong mode (e.g. Vpp off)
  BDM_RC_FLASH_PROGRAMING_BUSY   = 20,    // Busy with last Flash programming command
  BDM_RC_VDD_NOT_REMOVED         = 21,    // Target Vdd failed to fall
  BDM_RC_VDD_NOT_PRESENT         = 22,    // Target Vdd not present/failed to rise
  BDM_RC_VDD_WRONG_MODE          = 23,    // Attempt to cycle target Vdd when not controlled by BDM interface
};

#if (DEBUG&DEBUG_COMMANDS)
// Debugging sub commands
enum {
  BDM_DBG_ACKN             = 0,
  BDM_DBG_SYNC             = 1,
  BDM_DBG_TESTPORT         = 2,
  BDM_DBG_USBDISCONNECT    = 3,
  BDM_DBG_STACKSIZE        = 4,
  BDM_DBG_VPP_OFF          = 5,
  BDM_DBG_VPP_ON           = 6,
  BDM_DBG_FLASH12V_OFF     = 7,
  BDM_DBG_FLASH12V_ON      = 8,
  BDM_DBG_VDD_OFF          = 9,
  BDM_DBG_VDD3_ON          = 10,
  BDM_DBG_VDD5_ON          = 11,
  BDM_DBG_CYCLE_POWER      = 12,
  BDM_DBG_MEASURE_VDD      = 13,
};
#endif


// Capabilities for the hardware
enum {
  BDM_CAP_RESET        = (1<<0),   // RESET can be driven
  BDM_CAP_FLASH        = (1<<1),   // 12 V Flash programming supply available (RS08)
  BDM_CAP_VDDCONTROL   = (1<<2),   // Control over target Vdd
  BDM_CAP_VDDSENSE     = (1<<3),   // Sensing of target Vdd
};


#endif
