/*! \file
    \brief Minimal program to reboot USBDM into ICP mode

    \verbatim
    Copyright (C) 2008  Peter O'Donoghue

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Change History
   +====================================================================
   |   1 Jan 2009 | Updated for TBLCF                             - pgo
   |  20 Oct 2008 | Fixed size of SET_BOOT command                - pgo
   +====================================================================
    \endverbatim
*/
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "libusb.h"
#include "commands.h"


/* the device's vendor and product id */
#define OSBDM_VID (0x15A2) //!< OSBDM Vendor ID
#define OSBDM_PID (0x0021) //!< OSBDM Product ID

#define TBDML_VID (0x0425) //!< TBDML Vendor ID
#define TBDML_PID (0x1000) //!< TBDML Product ID

#define TBLCF_VID (0x0425) //!< Vendor ID for TBLCF
#define TBLCF_PID (0x1001) //!< Product ID for TBLCF

/* the device's endpoints */
#define EP_OUT (0x02)
#define EP_IN  (0x82)

#define TIMEOUT (500)   //!< Timeouts for BDM communications

#define BUF_SIZE (5)

typedef unsigned char  U8;
typedef unsigned short U16;
typedef unsigned int   U32;

usb_dev_handle *dev;    //!< USB device handle

/*! Opens a BDM device
 *
 * @return device handle for opened device or NULL on failure.
 */
usb_dev_handle *open_dev(void) {
   struct usb_bus *bus;
   struct usb_device *dev;

   for (bus = usb_get_busses(); bus; bus = bus->next) {
      for (dev = bus->devices; dev; dev = dev->next) {
         if (((dev->descriptor.idVendor  == OSBDM_VID) && (dev->descriptor.idProduct == OSBDM_PID)) ||
             ((dev->descriptor.idVendor  == TBDML_VID) && (dev->descriptor.idProduct == TBDML_PID)) ||
             ((dev->descriptor.idVendor  == TBLCF_VID) && (dev->descriptor.idProduct == TBLCF_PID))) {
            return usb_open(dev);
         }
      }
   }
   return NULL;
}

/*! Sends a message of up to 5 bytes to the TBDML/OSBDM device over EP0.
 * An immediate response is expected
 *
 * @param   usb_devh - device handle previous obtained from open_dev()
 * @param   data     - in/out buffer for transfer \n
 *
 * Entry:
 *       -  data[0]    = N, the number of bytes to receive from the device \n
 *       -  data[1]    = OSBDM command byte \n
 *       -  data[2..5] = parameter(s) for OSBDM command
 *
 * Exit: \n
 *       -  data[0]    = cmd response from OSBDM \n
 *       -  data[1..]  = data response from the device \n
 *
 * @return >0 => USB transfer OK, valid response from device \n
 *         0  => USB transfer Failed, or unexpected response from device
 */
unsigned char bdm_usb_recv_ep0( usb_dev_handle *usb_devh, U8 * data) {
   unsigned char count = data[0]+1;   /* data count is the first byte of the message */
                                      // +1 for possible debug byte
   unsigned char cmd   = data[1];     /* OSBDM/TBDML Command byte */
   int rc;
   if (usb_devh==NULL) {
      printf("USB EP0 receive request: device not open\r\n");
      return(1);
   }
   rc=usb_control_msg(usb_devh,
                      0xC0,                           // bmRequestType
                      cmd,                            // bRequest
                      (*(data+2))+256*(*(data+3)),    // wValue
                      (*(data+4))+256*(*(data+5)),    // wIndex
                      data,                           // ptr to data buffer
                      count,                          // wLength = # of data bytes
                      TIMEOUT                         // timeout
                      );
   if (rc<=0) {
      printf("USB EP0 receive: Transfer failed (USB error = %d)\r\n", rc);
      return 0;
   }

   if ((data[0] != 0) && (data[0] != cmd)) { // Hack to accept old error codes
      printf("USB EP0 receive (size = %d, recvd = %d):\r\n", count, rc);
      printf("Data=%X,%X,%X,%X,%X,\r\n",data[0],data[1],data[2],data[3],data[4]);
      return 0;
   }
   return rc;
}

/*! Sends a message of up to 5 bytes to the TBDML/OSBDM device over EP0.
 *  An immediate response is expected
 *
 * @param   usb_devh - device handle previous obtained from open_dev()
 * @param   data     - buffer for transfer \n
 *
 * Entry:
 *       -  data[0]      = N, the number of bytes to send to the device \n
 *       -  data[1]      = OSBDM command byte \n
 *       -  data[2..N-1] = parameter(s) for OSBDM command
 *
 * @return >0 => USB transfer OK, valid response from device \n
 *          0 => USB transfer Failed, or unexpected response from device
 */
unsigned char bdm_usb_send_ep0( usb_dev_handle *usb_devh, U8 * data) {
   unsigned char count = data[0];     /* data count is the first byte of the message */
                                      // +1 for possible debug byte?
   unsigned char cmd   = data[1];     /* OSBDM/TBDML Command byte */
   int rc;
   if (usb_devh==NULL) {
      printf("USB EP0 send request: device not open\r\n");
      return(1);
   }
   rc=usb_control_msg(usb_devh,
                      0x40,                           // bmRequestType
                      cmd,                            // bRequest
                      (*(data+2))+256*(*(data+3)),    // wValue
                      (*(data+4))+256*(*(data+5)),    // wIndex
                      data,                           // ptr to data buffer
                      (count>5)?(count-5):0,          // wLength = # of data bytes
                      TIMEOUT                         // timeout
                      );
   // Ignore errors since the BDM reboots before USB Tx is acknowledged.

   return 0;
}

/*! Connect to the BDM device

 * @return device handle for opened BDM device or NULL on failure.
 */
usb_dev_handle *OSBDMConnect(void){
   usb_dev_handle *dev;

   usb_init();         /* Initialise the library */
   usb_find_busses();  /* find all busses */
   usb_find_devices(); /* find all connected devices */

   dev = open_dev();
   if (dev == NULL) {
      return NULL;
   }

   if (usb_set_configuration(dev, 1) < 0) {
      usb_close(dev);
      return NULL;
   }

   if (usb_claim_interface(dev, 0) < 0) {
      usb_close(dev);
      return NULL;
   }
   return dev;
}

/*! Check & Report Firmware/Hardware version numbers
 *
 * @return \n
 *   <= 0  => Error or ICP not supported \n
 *   == 1  => V?.? Old version of ICP (re-program flash version) \n
 *   == 2  => V1.0 Old version of ICP (temporary reboot to ICP) \n
 *   == 3  => V1.1 Old version of ICP (temporary reboot to ICP) \n
 *   == 4  => V1.2 Newest! version of ICP (temporary reboot to ICP) \n

This has to handle several different version of the ICP code.  These are
indicated by different responses to the CMD_GET_VER command \n
1. Initial release (JB16 only) - no ICP version # \n
   Version response [8 bytes] \n
   -   [0]      status          = 0C \n
   -   [1]      BDM sw version  = 01 (v0.1) \n
   -   [2]      BDM hw version  = 01 (v0.1) \n
   -   [3..7]   ?,?,?,?,?       = garbage (due to bug in original USB code?) \n
   This uses a hard ICP mode where the flash is re-programmed to permanently
   force ICP on all further resets. \n
   The user is advised to update the entire Flash as the change in the ICP code
   is significant. \n

2. August 08 release (JB16 only) - ICP V1.0 \n
   Version response [6 bytes] \n
   -   [0]      status               = 0C        \n
   -   [1]      BDM sw version       = 10 (v1.0) \n
   -   [2]      BDM hw version       = 13 (v1.3) \n
   -   [3]      ICP sw version       = 10 (v1.0) \n
   -   [4]      ICP hw version       = 00 => JB16, 80 => JMxx \n
   -   [5]      Hardware identifier  = 01 to 06 indicates hardware type \n
   This uses a soft ICP mode where the flash is unchanged but the BDM is
   immediately reset into ICP mode.  This is temporary. \n
   The user is advised to update the entire Flash as the change in the ICP code
   is significant. \n

3. September 08 release (JB16 & JMxx & UF32) - ICP V1.1 \n
   Version response [5 bytes] \n
   -   [0]      status               = 0C        \n
   -   [1]      BDM sw version       = 10 (v1.0) \n
   -   [2]      Hardware identifier  = 01 to 08 indicates hardware type (in BDM code) \n
                                       +80 for JMxx versions \n
                                       +C0 for UF32 versions \n
   -   [3]      ICP sw version       = 11 (v1.1) \n
   -   [4]      Hardware identifier  = 01 to 08 indicates hardware type (in ICP code) \n
                                      +80 for JMxx versions \n
                                      +C0 for UFxx versions \n
   As for version 2 but the hardware identification is kept in the ICP code and
   the BDM code.  These should always match!  This allows the JMxx driver to
   automatically update the BDM flash with the correct version of the Firmware
   even if the BDM flash area is corrupted. \n
   The user is advised to update the entire Flash as the change in the ICP code
   is significant. \n
4. October 08 release (JB16 & JMxx & UF32) - ICP V1.2 \n
   Version response [5 bytes] \n
   -   [0]      status               = 0C        \n
   -   [1]      BDM sw version       = 10 (v1.0) \n
   -   [2]      Hardware identifier  = 01 to 08 indicates hardware type (in BDM code) \n
                                       +80 for JMxx versions \n
                                       +C0 for UF32 versions \n
   -   [3]      ICP sw version       = 12 (v1.2) \n
   -   [4]      Hardware identifier  = 01 to 0F indicates hardware type (in ICP code) \n
                                      +80 for JMxx versions \n
                                      +C0 for UFxx versions \n
   As for version 3 but fixes a bad bug in the JB16 ICP code!
   The user is advised to only update the BDM flash area as the ICP is current. \n
 */
//=================================================================
int OSBDMReportVersion(usb_dev_handle *usb_devh){
static const char *BDMCPUTypes[]={"JB16","????","JM16","UF32"};
static const char *hardwareDescription[] = {
   "Unknown or invalid hardware",                  // 0
   "USBDM - Extended TBDML/OSBDM (JB16DWE)",       // 1
   "TBDML - Minimal TBDML",                        // 2
   "TBDML - Minimal TBDML (Internal version)",     // 3
   "OSBDM - Original OSBDM",                       // 4
   "Witztronics TBDML/OSBDM",                      // 5
   "OSBDM+ - Extended OSBDM (RS08 support)",       // 6
   "USBDM - Extended TBDML/OSBDM (JMxxCLD)",       // 7
   "USBDM - Extended TBDML/OSBDM (JMxxCLC)",       // 8
   "USBSPYDER08 - SofTec USBSPYDER08      ",       // 9
   "USBDM - Extended TBDML/OSBDM (UF32PBE)",       // 10
   "TBLCF - Extended TBLCF (JMxxCLD)",             // 11
   "USBDM/CF - Combined USBDM/TBLCF (JMxxCLD)",    // 12
   "Reserved #3",                                  // 13
   "Reserved #4",                                  // 14
   "Custom",                                       // 15
   };

   // Get OSBDM S/W version, H/W Type
   U8 getVersionCmd[10]   = {7,CMD_GET_VER,0,0,0,0,0,0,0,0};
   int size;
   int i;

   U8 bdmVersion;
   U8 icpVersion;
   U8 hardwareVersion; // Obsolete
   U8 hardwareIdentification;

   size = bdm_usb_recv_ep0(usb_devh, getVersionCmd);

   if (size < 3) {
      printf("Error: USB receive failed (size = %d)\n", size);
      return -1;
   }

   printf("Size = %d, Data = ", size);

   for (i=0; i<size; i++)
      printf("%2.2X,", getVersionCmd[i]);
   printf("\n");

   bdmVersion      = getVersionCmd[1];
   hardwareVersion = getVersionCmd[2];
   printf("TBDML/OSBDML Software Version = %d.%d\n",
             (bdmVersion>>4)&0x0F,bdmVersion&0x0F);

   printf("TBDML/OSBDML Hardware Version = %d.%d\n",
          (hardwareVersion>>4)&0x0F,hardwareVersion&0x0F);

   if ((bdmVersion < 0x10) || (size == 8)) { // Early version, maybe old ICP
      printf("No hardware description available\n");
      return 1;
   }

   icpVersion = getVersionCmd[3];

   if (bdmVersion == 0xFF) { // indicates in ICP mode on JMxx
      printf("***\nWarning: BDM appears to be already in ICP mode\n***\n");
      hardwareIdentification = getVersionCmd[2];
   }
   else if (icpVersion == 0x10)
      hardwareIdentification = getVersionCmd[5];
   else {
      hardwareIdentification = getVersionCmd[4];
      if (hardwareVersion != hardwareIdentification)
         printf("***\nWarning: BDM Software and ICP software don't agree on hardware type!!\n***\n");
   }

   printf("TBDML/OSBDML ICP Version      = %d.%d\n",
          (icpVersion>>4)&0x0F,icpVersion&0x0F);

   if ((hardwareIdentification & 0x3F) >
       sizeof(hardwareDescription)/sizeof(hardwareDescription[0])) {
      hardwareIdentification = 0;
   }

   printf("TBDML/OSBDML Hardware         = %s - %s\n",
          BDMCPUTypes[(hardwareIdentification>>6)&0x03],
          hardwareDescription[hardwareIdentification&0x3F]);

   if (icpVersion == 0x10)
      return 2;
   if (icpVersion == 0x11)
      return 3;
   // ICP V# > 1.1
   return 4;
}

/*! Set for flash load on next reboot
 */
int OSBDMSetBoot(usb_dev_handle *usb_devh){
   U8 setBootCmd[]      = {5,CMD_SET_BOOT,'B','O','O','T'};

   // Ignore errors
   bdm_usb_send_ep0(usb_devh, (char*)setBootCmd);

   printf("setBootCommand Complete\n");

   return 0;
}


int main(void) {
   char commandBuffer[100];
   int ICP_Version;

   puts("\n\n\n\n");

   dev = OSBDMConnect();
   if (dev == NULL) {
      printf("Error: No USBDM device found!\n");
      puts("Press ENTER to continue\n");
      gets(commandBuffer);
      return 0;
   }

   ICP_Version = OSBDMReportVersion(dev);

   if (ICP_Version <= 0) {
      puts("Press ENTER to continue\n");
      gets(commandBuffer);
      goto cleanUp;
   }

   if (ICP_Version == 1) {
      puts("\n**********************************************");
      puts("**********************************************");
      puts("******                                  ******");
      puts("******            WARNING               ******");
      puts("******                                  ******");
      puts("******   Setting ICP mode will make     ******");
      puts("******   the BDM unusable until it      ******");
      puts("******     has been re-programmed       ******");
      puts("******     by using the Freescale       ******");
      puts("******          ICP software.           ******");
      puts("******                                  ******");
      puts("******   Remove and replace the BDM.    ******");
      puts("******   before trying to program it    ******");
      puts("******     using the Freescale ICP      ******");
      puts("******            software.             ******");
      puts("**********************************************\n");
      puts("**********************************************\n");
      puts("Permanently set ICP mode on TBDML/OSBDM device (yes/No)?");
   }
   else {
      puts("\n**********************************************");
      puts("**********************************************");
      puts("******                                  ******");
      puts("******           ADVICE                 ******");
      puts("******                                  ******");
      puts("******   This will temporarily reboot   ******");
      puts("******      the BDM into ICP mode.      ******");
      puts("******   No permanent changes are made  ******");
      puts("******         to the BDM.              ******");
      puts("******                                  ******");
      puts("******   Do not remove the BDM until    ******");
      puts("******    you have re-programmed it     ******");
      puts("******     using the Freescale ICP      ******");
      puts("******            software.             ******");
      puts("******                                  ******");
      puts("**********************************************");
      puts("**********************************************\n");
      puts("Temporarily reboot TBDML/OSBDM device into ICP mode (yes/No)?");
   }

   gets(commandBuffer);
   if ((commandBuffer[0] == 'y')||(commandBuffer[0] == 'Y'))
      OSBDMSetBoot(dev);

   if (ICP_Version < 4) {
      puts("\n**********************************************");
      puts("******   Old ICP software was found.    ******");
      puts("******                                  ******");
      puts("******  This ICP version has a BUG!     ******");
      puts("******                                  ******");
      puts("******  You are advised to update ALL   ******");
      puts("******   of the Flash (Bulk erase).     ******");
      puts("******  (Doesn't apply to JM60 or UF32) ******");
      puts("******                                  ******");
      puts("**********************************************");
      puts("Press Enter to continue.");
      gets(commandBuffer);
   }
   else {
      puts("Press Enter to continue.");
      gets(commandBuffer);
   }

cleanUp:
   if (dev != NULL) {
      usb_release_interface(dev, 0);
      usb_close(dev);
   }

  return 0;
}
